@@ -6,14 +6,15 @@ import sys
 from collections import OrderedDict, defaultdict
 from contextlib import contextmanager
 import textwrap
-from .adapters import BACK, FRONT, PREFIX, SUFFIX, ANYWHERE, LINKED
-from .modifiers import *
-from .filters import *
+from atropos.adapters import BACK, FRONT, PREFIX, SUFFIX, ANYWHERE, LINKED
+from atropos.modifiers import *
+from atropos.filters import *
 
 # TODO: Would be good to have a more generic way for filters
 # and modifiers to report summary information
 # TODO: Rewrite this to use a templating engine.
 # TODO: Need a better way to line things up in columns.
+# TODO: https://github.com/marcelm/cutadapt/issues/128
 
 def check_equal_merger(dest, src):
     assert dest == src
@@ -271,7 +272,7 @@ class Summary(object):
         
         return stats
 
-def print_report(paired, options, wallclock_time, cpu_time, stats, trimmer_classes):
+def print_report(options, wallclock_time, cpu_time, stats, trimmer_classes):
     outfile = options.report_file
     close = False
     
@@ -283,6 +284,7 @@ def print_report(paired, options, wallclock_time, cpu_time, stats, trimmer_class
     else:
         return
     
+    paired = options.paired
     stats["paired"] = paired
     stats["wallclock_time"] = max(wallclock_time, 0.01)
     stats["cpu_time"] = max(cpu_time, 0.01)
