@@ -5,6 +5,7 @@ from atropos.seqio import Sequence
 from atropos.modifiers import *
 from atropos.adapters import *
 from atropos.align import MatchInfo
+from atropos.util import reverse_complement as rc
 
 DUMMY_ADAPTER = Adapter("ACGT", FRONT)
 def front_match(read):
@@ -178,7 +179,7 @@ def test_Swift_trimmer():
     trimmed = trimmer(read1, read2)
     assert trimmed[0].sequence == seq[:-10]
     assert trimmed[1].sequence == seq[10:]
-
+                            
 def test_overlapping():
     trimmer = MergeOverlapping(min_overlap=10, error_rate = 0.1)
     a1 = 'AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTC'
@@ -245,6 +246,20 @@ def test_overlapping():
     assert read1_merged.merged is False
     assert read2 is not None
 
+def test_overlapping_with_error_correction():
+    trimmer = MergeOverlapping(min_overlap=10, error_rate = 0.1, mismatch_action='liberal')
+    r1 = 'AGATCGGAAGACCGTCATGTAGGGAAAGAGTGTAGATCTC'
+    q1 = 'FFFFFFFFFFF#FFFFFFFFFFFFFFFFFFFFF#######'
+    r2 = reverse_complement('AGATCGGTAGAGCGTCGTGTAGGGAAATAGTGTAGATCTC')
+    q2 = ''.join(reversed('FFFFFFFFFFFFFFFF#FFFFFFFFFF#FFFFFFFFFFFF'))
+    read1 = Sequence('foo', r1, q1)
+    read2 = Sequence('foo', r2, q2)
+    read1_merged, read2_merged = trimmer(read1, read2)
+    assert read1_merged.merged
+    assert read2_merged is None
+    assert read1_merged.sequence == 'AGATCGGTAGAGCGTCATGTAGGGAAAGAGTGTAGATCTC'
+    assert read1_merged.qualities == 'FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF#######'
+
 def test_mismatched_adapter_overlaps():
     """
     This is a test case from real data. The adapter overlaps 1 less bp
@@ -276,7 +291,7 @@ def test_mismatched_adapter_overlaps():
     
 def test_error_correction():
     a1 = 'AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTC'
-    a2 = reverse_complement('AGATCGGAAGAGCACACGTCTGAACTCCAGTCACGAGTTA')
+    a2 = 'AGATCGGAAGAGCACACGTCTGAACTCCAGTCACGAGTTA'
     frag = 'CCAAGCAGACATTCACTCAGATTGCA'
     correct_frag = 'CCAAGTAGACATTCGCTCAGATTGCA'
     
@@ -295,8 +310,9 @@ def test_error_correction():
     q2 = ['#'] * 40
     # quality of read2 > quality of read1 at pos 11
     q2[len(frag)-15] = 'A'
-    r2 = reverse_complement(a2 + ''.join(r2))[0:40]
+    r2 = reverse_complement(reverse_complement(a2) + ''.join(r2))[0:40]
     q2 = ''.join(q2)
+    
     read1 = Sequence('foo', r1, q1)
     read2 = Sequence('foo', r2, q2)
     
@@ -304,10 +320,143 @@ def test_error_correction():
     adapter1 = parser.parse(a1)
     adapter2 = parser.parse(a2)
     
-    cutter = InsertAdapterCutter(adapter1, adapter2, mismatch_action='best')
+    cutter = InsertAdapterCutter(adapter1, adapter2, mismatch_action='liberal')
     
     new_read1, new_read2 = cutter(read1, read2)
     assert len(new_read1) == 26
+    assert new_read1.insert_overlap
     assert new_read1.sequence == correct_frag
     assert len(new_read2) == 26
+    assert new_read2.insert_overlap
     assert new_read2.sequence == reverse_complement(correct_frag)
+
+def test_error_correction_no_insert_match_one_adapter_match():
+    a1 = 'AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTC'
+    a2 = 'AGATCGGAAGAGCACACGTCTGAACTCCAGTCACGAGTTA'
+    a2_mod = 'ACATCGGAAGAGCACACGTCTGAACTCCAGTCACGAGTTA'
+    frag = 'CCAAGCAGACATTCACTCAGATTGCA'
+    correct_frag = 'CCAAGTAGACATTCGCTCAGATTGCA'
+    
+    r1 = list(frag)
+    # C>T at pos 6
+    r1[5] = 'T'
+    q1 = ['#'] * 40
+    # quality of read1 > quality of read2 at pos 6
+    q1[5] = 'A'
+    r1 = (''.join(r1) + a1)[0:40]
+    q1 = ''.join(q1)
+    
+    r2 = list(frag)
+    # A>G at pos 15
+    r2[14] = 'G'
+    q2 = ['#'] * 40
+    # quality of read2 > quality of read1 at pos 11
+    q2[len(frag)-15] = 'A'
+    r2 = reverse_complement(reverse_complement(a2_mod) + ''.join(r2))[0:40]
+    q2 = ''.join(q2)
+    
+    read1 = Sequence('foo', r1, q1)
+    read2 = Sequence('foo', r2, q2)
+    
+    parser1 = AdapterParser()
+    adapter1 = parser1.parse(a1)
+    # Allow zero mismatches to prevent adapter alignment
+    parser2 = AdapterParser(max_error_rate=0)
+    adapter2 = parser2.parse(a2)
+    
+    # Allow zero mismatches to prevent insert alignment
+    cutter = InsertAdapterCutter(
+        adapter1, adapter2, mismatch_action='liberal',
+        max_insert_mismatch_frac=0)
+    
+    new_read1, new_read2 = cutter(read1, read2)
+    assert len(new_read1) == 26
+    assert not new_read1.insert_overlap
+    assert new_read1.sequence == correct_frag
+    assert len(new_read2) == 26
+    assert not new_read2.insert_overlap
+    assert new_read2.sequence == reverse_complement(correct_frag)
+
+def test_error_correction_no_insert_match_two_adapter_matches():
+    a1 = 'AGATCGGAAGAGCGTCGTGTAGGGAAAGAGTGTAGATCTC'
+    a2 = 'AGATCGGAAGAGCACACGTCTGAACTCCAGTCACGAGTTA'
+    frag = 'CCAAGCAGACATTCACTCAGATTGCA'
+    correct_frag = 'CCAAGTAGACATTCGCTCAGATTGCA'
+    
+    r1 = list(frag)
+    # C>T at pos 6
+    r1[5] = 'T'
+    q1 = ['#'] * 40
+    # quality of read1 > quality of read2 at pos 6
+    q1[5] = 'A'
+    r1 = (''.join(r1) + a1)[0:40]
+    q1 = ''.join(q1)
+    
+    r2 = list(frag)
+    # A>G at pos 15
+    r2[14] = 'G'
+    q2 = ['#'] * 40
+    # quality of read2 > quality of read1 at pos 11
+    q2[len(frag)-15] = 'A'
+    r2 = reverse_complement(reverse_complement(a2) + ''.join(r2))[0:40]
+    q2 = ''.join(q2)
+    
+    read1 = Sequence('foo', r1, q1)
+    read2 = Sequence('foo', r2, q2)
+    
+    parser = AdapterParser()
+    adapter1 = parser.parse(a1)
+    adapter2 = parser.parse(a2)
+    
+    # Allow zero mismatches to prevent insert alignment
+    cutter = InsertAdapterCutter(
+        adapter1, adapter2, mismatch_action='liberal',
+        max_insert_mismatch_frac=0)
+    
+    new_read1, new_read2 = cutter(read1, read2)
+    assert len(new_read1) == 26
+    assert not new_read1.insert_overlap
+    assert new_read1.sequence == correct_frag
+    assert len(new_read2) == 26
+    assert not new_read2.insert_overlap
+    assert new_read2.sequence == reverse_complement(correct_frag)
+
+def ints2quals(ints):
+    return ''.join(chr(i+33) for i in ints)
+
+def test_overwrite_read():
+    overwrite = OverwriteRead(20, 40, 10)
+    lowseq = 'ACGT' * 5
+    highseq = 'TCAG' * 5
+    
+    # mean lowq > 20, mean highq > 40
+    lowq = (11, 31, 16, 24, 16, 20, 17, 19, 21, 28) * 2
+    highq = (22, 62, 32, 48, 32, 40, 34, 38, 42, 56) * 2
+    read1 = Sequence('foo', lowseq, ints2quals(lowq))
+    read2 = Sequence('foo', highseq, ints2quals(highq))
+    new_read1, new_read2 = overwrite(read1, read2)
+    assert new_read1.sequence == lowseq
+    assert new_read1.qualities == ints2quals(lowq)
+    assert new_read2.sequence == highseq
+    assert new_read2.qualities == ints2quals(highq)
+    assert new_read1.corrected == new_read2.corrected == 0
+    
+    # mean lowq < 20, mean highq > 40
+    lowq = tuple(i-1 for i in lowq)
+    read1 = Sequence('foo', lowseq, ints2quals(lowq))
+    new_read1, new_read2 = overwrite(read1, read2)
+    assert new_read1.sequence == rc(highseq)
+    assert new_read1.qualities == ints2quals(reversed(highq))
+    assert new_read2.sequence == highseq
+    assert new_read2.qualities == ints2quals(highq)
+    assert new_read1.corrected == new_read2.corrected == 1
+    
+    # mean lowq < 20, mean highq < 40
+    highq = tuple(i-1 for i in highq)
+    read2 = Sequence('foo', highseq, ints2quals(highq))
+    new_read1, new_read2 = overwrite(read1, read2)
+    assert new_read1.sequence == lowseq
+    assert new_read1.qualities == ints2quals(lowq)
+    assert new_read2.sequence == highseq
+    assert new_read2.qualities == ints2quals(highq)
+    assert new_read1.corrected == new_read2.corrected == 0
